package name.ianmorgan.canfactory.exam;

import java.util.Date;

import name.ianmorgan.canfactory.exam.ExamResults;

import org.junit.Assert;
import org.junit.Test;

public class ExamResultsTest {

	private ExamResults results;

	@Test
	public void shouldReturnNumberOfGradeAResults() {
		// note grade A have scores of 70 or above
		Assert.assertEquals(2, buildResults(new int[] { 69, 70, 71 }).numberOfGradeAResults());
	}

	@Test
	public void shouldReturnNumberOfGradeBResults() {
		// note grade B have scores of 60 or more and below 70
		Assert.assertEquals(2, buildResults(new int[] { 59, 60, 69, 70 }).numberOfGradeBResults());
	}

	@Test
	public void shouldReturnNumberOfGradeCResults() {
		// note grade C have scores of 50 or more and below 60
		Assert.assertEquals(2, buildResults(new int[] { 49, 50, 59, 60 }).numberOfGradeCResults());
	}

	@Test
	public void shouldReturnNumberOfPasses() {
		// note passes are any grade A,B or C
		Assert.assertEquals(3, buildResults(new int[] { 40, 50, 60, 70 }).numberOfPasses());
	}

	@Test
	public void shouldReturnNumberOfFails() {
		// note fails are any that don't pass!
		Assert.assertEquals(1, buildResults(new int[] { 40, 50, 60, 70 }).numberOfFails());
	}

	@Test
	public void shouldReturnNumberOfDistinctScores() {
		Assert.assertEquals(2, buildResults(new int[] { 1, 99, 1 }).numberOfDistinctResults());
	}

	@Test
	public void shouldNeverDistibuteScoreToMoreThanOneCategory() {
		// quick sanity check for mismatched boundary conditions
		int[] allPossibleScores = new int[101];
		for (int i = 0; i <= 100; i++) {
			allPossibleScores[i] = i;
		}
		results = buildResults(allPossibleScores);

		Assert.assertEquals(101, results.numberOfFails() + results.numberOfPasses());
		Assert.assertEquals(results.numberOfPasses(), results.numberOfGradeAResults() + results.numberOfGradeBResults()
				+ results.numberOfGradeCResults());
	}

	private ExamResults buildResults(int[] scores) {
		return new ExamResults("code", new Date(), scores);
	}
	
	@Test
	public void shouldReallyWriteSomeCachingTests() {
		// call everything thing twice to ensure the internal caching logic is
		// exercised and doesn't unwittingly overwrite cached values.
	}

}
